const
NCounters = 8;

{Port offsets from 4I36 BASE port}
IndexOffset       = $00;
CounterLowOffset  = $02;
CounterHighOffset = $04;
CounterContOffset = $06;
PortAOffset       = $08;
PortADDROffset    = $0A;
PortBOffset       = $0C;
PortBDDROffset    = $0E;

{ Index register bits (bits 0 to 2 are counter channel select bits)}

IDXConfigEna  = $8000;      { must be set to enable EEPROM re-config }
IDXAutoInc    = $4000;      { if set, reading counter high increments index }
IDXGlobalClr  = $2000;      { if set clears all counters }
IDXGlobalHold = $1000;      { if set holds all counters }
CHSelMask     = $0007;

{Counter control register bits (per counter) }
GlobalClear   = $2000;      { same as in index reg }
GlobalHold    = $1000;      { same as in index reg }

AutoCount     = $800;       { if set, counter counts up at 48 MHz}
CounterMode   = $400;       { if set counter is in up down mode (1X)}
                            { otherwise quadrature mode (4X) }
QuadFilter    = $200;       { use slow input digital filter if set }
                            { this limits input frequency to about 1.5 MHz }
                            { and quad count rate to about 6 MHz }
LocalHold     = $100;       { holds count when set }
IndexGate     = $80;        { if set, quada and quadb must be low to detect index}
ClearOnce     = $40;        { if set, ClearOnIndex is cleared once index is detected }
ClearOnIndex  = $20;        { if set, index edge will clear counter }
IndexPolarity = $10;        { sets polarity of active index edge }
LatchOnRead   = $08;        { if set counter is latched on read from host }
IndexClear    = $04;        { read = real time index signal, if set on write, clears counter }
QuadB         = $02;        { read only - real time quadb input}
QuadA         = $01;        { read only - real time quada input}

RefRate = 48e6; { the reference count rate when a counter is used as a timer }

var
Index : word;
CountLow : word;
CountHigh : word;
CounterCont : word;
PortA : word;
PortADDR : word;
PortB : word;
PortBDDR : word;

I36BasePort : word;

type LongIntWordRec = record
  LowWord  : word;
  HighWord : word;
end;

procedure Init4I36;
var counters : word;
  begin
  { first initialize our port addresses soe's we dont gotta do the add every time }
  Index := I36BasePort + IndexOffset;
  CountLow := I36BasePort + CounterLowOffset;
  CountHigh := I36BasePort + CounterHighOffset;
  CounterCont := I36BasePort + CounterContOffset;
  PortA := I36BasePort + PortAOffset;
  PortADDR := I36BasePort + PortADDROffset;
  PortB := I36BasePort + PortBOffset;
  PortBDDR := I36BasePort + PortBDDROffset;
  portw[Index] := IDXGlobalClr;
  portw[Index] := $0000;
  for counters := 0 to 7 do
  begin
    portw[Index] := counters;
    portw[CounterCont]  := LatchOnRead;
  end;
end;

function ReadCounter(ch : byte): longint;
var
tlong : longint;
begin
  portw[Index] := ch and ChSelMask;
  LongIntWordRec(tlong).LowWord  := portw[CountLow];
  LongIntWordRec(tlong).HighWord := portw[CountHigh];
  ReadCounter := tlong;
end;

procedure ClearCounter(ch : byte);
var oldccr : word;
begin
  portw[Index] := ch and ChSelMask;
  oldccr := portw[CounterCont];
  portw[CounterCont] := oldccr or IndexClear;
end;

procedure FilterOn(ch : byte);
var oldccr : word;
begin
  portw[Index] := ch and ChSelMask;
  oldccr := portw[CounterCont];
  portw[CounterCont] := (oldccr and ($FFFF xor IndexClear)) or QuadFilter;
end;

procedure FilterOff(ch : byte);
var oldccr : word;
begin
  portw[Index] := ch and ChSelMask;
  oldccr := portw[CounterCont];
  portw[CounterCont] := (oldccr and ($FFFF xor IndexClear)) and ($FFFF xor QuadFilter);
end;

procedure LatchCounter(ch : byte); { latch an individual counter }
begin                              { only works if LatchOnRead not set on this counter }
  portw[Index] := ch and ChSelMask;
  portw[CountLow] := $0000;
end;

procedure LatchAllCounters;        { latch all counters that }
begin                              { do not have LatchOnRead set }
  portw[CountHigh] := $0000;
end;
